package de.ruben.opencms.modules.ftpd;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.URI;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import org.opencms.file.CmsFile;
import org.opencms.file.CmsObject;
import org.opencms.file.CmsProject;
import org.opencms.file.CmsProperty;
import org.opencms.file.CmsResource;
import org.opencms.file.CmsResourceFilter;
import org.opencms.file.types.CmsResourceTypeBinary;
import org.opencms.file.types.CmsResourceTypeImage;
import org.opencms.file.types.CmsResourceTypeJsp;
import org.opencms.file.types.CmsResourceTypePlain;
import org.opencms.main.CmsException;

import de.ruben.opencms.modules.ftpd.Logger;
import de.ruben.opencms.modules.ftpd.ServerPI;


/*
 * 
 * author ruben malchow (ruben@khm.de), see http://disco.kalkatraz.de/opencmsftpd
 * for details. 
 * 
 */

public class FtpFile extends File {

	private CmsResource cr;
	private CmsProject cp;
	private CmsObject co;
	
	private String path;
	
	private boolean isRoot=false;
	private FtpFile realFile;
	
	
	public FtpFile(CmsObject co,CmsProject cp,CmsResource cr) {
		super(ServerPI.normalizePath('/'+cp.getName()+'/'+cr.getRootPath()));
		this.path=ServerPI.normalizePath('/'+cp.getName()+'/'+cr.getRootPath());
		this.cr=cr;
		this.cp=cp;
		this.co=co;
		//Logger.log(Logger.LOG_DEBUG,"FtpFile.<init> : cr is "+(cr!=null?cr.getClass().getName():"null"));
	}

	public FtpFile(FtpFile ff) {
		super(ff.getAbsolutePath()+Server.getSuffix());
		realFile=ff;
		cp=ff.getCmsProject();
		//Logger.log(Logger.LOG_DEBUG,"FtpFile.<init> : path is \""+getPath()+"\"");
	}

	public FtpFile(CmsObject co,String path) {
		super(path);
		path=ServerPI.normalizePath(path);
		Logger.log(Logger.LOG_DEBUG,"FtpFile.<init> : path is \""+path+"\"");
		this.co=co;
		if (path.length()==1) {
			isRoot=true;
		} else {
			this.path=path;
			ensureRessource();
		}
	}

	private void tryResource() {
		co.getRequestContext().setCurrentProject(cp);
		try {
			cr=co.readResource(ServerPI.normalizePath(path));
		} catch (Exception e) {
		}
	}
	
	private void ensureRessource() {
		StringTokenizer st=new StringTokenizer(path,"/");
		String pname=st.nextToken();
		try {
			List l=co.getAllAccessibleProjects();
			for (Iterator iter = l.iterator(); iter.hasNext();) {
				CmsProject element = (CmsProject) iter.next();
				if(element.getName().compareTo(pname)==0) {
					this.cp=element;
					this.path=path.substring(element.getName().length()+1,path.length())+"/";
					this.path=ServerPI.normalizePath(this.path);
					
					//Logger.log(Logger.LOG_DEBUG,"FtpFile.ensureRessource() : "+this.path+" ends with "+Server.getSuffix()+"? ---> " +(this.path.endsWith(Server.getSuffix())));

					if(Server.getSuffix().length()>0 && this.path.endsWith(Server.getSuffix())) {
						String pathTmp=
							this.path.substring(
									0,
									this.path.length()-Server.getSuffix().length()
							);
						realFile=new FtpFile(co,'/'+cp.getName()+'/'+pathTmp);
					} else {
						tryResource();
					}
					
					//Logger.log(Logger.LOG_DEBUG,"FtpFile.ensureRessource() : path is "+this.path+", project is "+cp.getName());
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public FtpFile(String arg0, String arg1) {
		super(arg0, arg1);
	}

	public FtpFile(File arg0, String arg1) {
		super(arg0, arg1);
	}

	public FtpFile(URI arg0) {
		super(arg0);
	}

	public String getCmsPath() {
		return isRoot?null:cr.getRootPath();
	}
	
	public CmsProject getCmsProject() {
		return cp;
	}
	
	public String getAbsolutePath() {
		return getPath();
	}

	public String getCanonicalPath() throws IOException {
		return getPath();
	}

	public String getPath() {
		String path="/";
		if(realFile!=null) {
			path=realFile.getAbsolutePath()+Server.getSuffix();
		} else {
			if (!isRoot) {
				path=path+cp.getName()+"/"+getCmsPath();
			}
		}
		return ServerPI.normalizePath(path);
	}

	public boolean isDirectory() {
		if(realFile!=null) return false;
		return isRoot?true:cr.isFolder();
	}

	public boolean isFile() {
		if(realFile!=null) return true; 
		return isRoot?false:!cr.isFolder();
	}

	public String[] list() {
		File[] files=listFiles();
		String[] out=new String[files.length];
		for (int i = 0; i < files.length; i++) {
			out[i]=files[i].getName();
		}
		return out;
	}

	public String[] list(FilenameFilter arg0) {
		return list(null);
	}

	public File[] listFiles() {
		if(isDirectory()) {
			try {
				co.getRequestContext().setCurrentProject(cp);
				List l=co.getResourcesInFolder(cr.getRootPath(),CmsResourceFilter.DEFAULT);

				File[] out=null;
				
				if(Server.getSuffix().length()>0) {
					out=new File[l.size()*2];
				} else {
					out=new File[l.size()];
				}
				

				int i=0;
				for(Iterator iter=l.iterator();iter.hasNext();) {
					
					
					CmsResource crTmp=(CmsResource)iter.next();
					FtpFile ff=new FtpFile(co,cp,crTmp);
					//System.err.println("-------------> "+i+" : "+crTmp);
					
					out[i]=ff;
					i++;

					if(Server.getSuffix().length()>0) {
						FtpFile fff=new FtpFile(ff);
						out[i]=fff;
						i++;
					}
				}
				return out;
			} catch (CmsException e) {
				e.printStackTrace();
			}
			
		}
		return new File[0];
	}

	public File[] listFiles(FileFilter arg0) {
		return listFiles();
	}

	public File[] listFiles(FilenameFilter arg0) {
		return listFiles();
	}

	public long length() {
		try {
			if(cr==null) return getContent().length;
			return Math.max(0,cr.getLength());
		} catch (Exception e) {
			// TODO: handle exception
		}
		return 0;
	}

	public boolean exists() {
		if(isRoot) return true;
		if(realFile!=null) return realFile.exists();
		return cr!=null;
	}
	
	private FtpFile getRealFile() {
		if(realFile==null) return null;
		try {
			co.getRequestContext().setCurrentProject(cp);
			CmsResource crTmp=null;
			try {
				co.readResource(realFile.getCmsPath());
			} catch (Exception e) {
			}
			realFile = new FtpFile(co,cp,crTmp);
		} catch (Exception e) {
		}
		return realFile;
	}
	
	public void setProperties(Properties p) throws CommandException {
		try {
			tryResource();
			co.getRequestContext().setCurrentProject(cp);
			
			boolean newFile=false;

			if(!exists()) {
				try {
					int typeId=Integer.parseInt(""+p.get("resourceTypeId"));
					this.cr=co.createResource(path,typeId);
					newFile=true;
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			
			CmsFile cf=null;
			
			if(cr.isFile()) {
				cf=co.readFile(cr.getRootPath());
				co.getLock(cf);
			}
			
			co.lockResource(cr.getRootPath());
			
			Hashtable h=new Hashtable();
			
			for (Iterator iter = p.keySet().iterator(); iter.hasNext();) {
				String element = (String) iter.next();
				if(element.compareTo("resourceTypeId")==0) {
					try {
						int typeId=Integer.parseInt(""+p.get(element));
						if(!newFile && typeId!=cr.getTypeId()) {
							Logger.log(Logger.LOG_DEBUG,"Resource Type has changed on "+cr.getRootPath()+", setting to new type "+typeId+" (from "+cr.getTypeId()+")");
							cf.setType(typeId);
							if(isFile()) {
								co.writeFile(cf);
							}
						}
					} catch (Exception e) {
						e.printStackTrace();
						throw new CommandException(500, "could not set resource type id ("+e.getMessage()+")");
					}
				} else if(element.compareTo("resourceId")==0) {
					
				} else {
					String pName=null;
					String rValue=null;
					String sValue=null;
					if(element.startsWith("resource.")) {
						pName=element.substring("resource.".length());
					} else if(element.startsWith("structure.")) {
						pName=element.substring("structure.".length());
					}
					if(pName!=null) {
						rValue=p.getProperty("resource."+pName);
						sValue=p.getProperty("structure."+pName);
						CmsProperty cProp=new CmsProperty(pName,sValue,rValue);
						h.put(pName,cProp);
					}
				}
			}
			
			Vector v=new Vector();
			v.addAll(h.values());
			
			co.writePropertyObjects(cr.getRootPath(),v);
		} catch (CommandException ce) {
			throw ce;
		} catch (Exception ce) {
			ce.printStackTrace();
			throw new CommandException(500,"could not write to file ("+path+") : "+ce.getMessage()+")");
		} finally {
			try {
				co.unlockResource(cr.getRootPath());
			} catch (Exception e) {
			}
		}
		
	}
	
	public void setContent(byte[] bytes) throws CommandException {
		if(realFile!=null) {
			Properties p=new Properties();
			ByteArrayInputStream bais=new ByteArrayInputStream(bytes);
			try {
				p.load(bais);
				getRealFile().setProperties(p);
			} catch (CommandException ce) {
				throw ce;
				/*
				} catch (InvalidPropertiesFormatException e) {
					e.printStackTrace();
					throw new CommandException(500,"Invalid properties file format ("+e.getMessage()+")");
				*/
			} catch (IOException e) {
				e.printStackTrace();
				throw new CommandException(500,"Error reading contents of file ("+e.getMessage()+")");
			}
			return;		
				
		} else if(!isFile()) {
			throw new RuntimeException("not a file");
		} else {
			try {
				CmsFile cf;
				co.getRequestContext().setCurrentProject(cp);
				
				cf = co.readFile(cr.getRootPath());
				
				co.lockResource(cr.getRootPath());
				co.getLock(cf);

				cf.setContents(bytes);
				try {
					co.writeFile(cf);
				} catch (Exception e) {
					e.printStackTrace();
				}
			} catch (Exception ce) {
				ce.printStackTrace();
				throw new RuntimeException("could not write to file ("+ce.getMessage()+")");
			} finally {
				try {
					co.unlockResource(cr.getRootPath());
				} catch (Exception e) {
				}
			}
		}
	}

	public byte[] getContent() {
		if(realFile!=null) {
			CmsResource crTmp=realFile.cr;
			try {
				co.getRequestContext().setCurrentProject(cp);
				Properties p=new Properties();
				p.put("resourceTypeId",realFile.cr.getTypeId()+"");
				p.put("resourceId",realFile.cr.getResourceId()+"");
				List lSiblings=co.readSiblings(crTmp.getRootPath(),CmsResourceFilter.ALL);
				String sSiblings="";
				for (Iterator iter = lSiblings.iterator(); iter.hasNext();) {
					CmsResource element = (CmsResource) iter.next();
					sSiblings=sSiblings+element.getRootPath()+(iter.hasNext()?";":"");
				}
				p.put("siblingOf",sSiblings);
				
				List lProperties=realFile.co.readPropertyObjects(crTmp.getRootPath(),false);
				
				for (Iterator iter = lProperties.iterator(); iter.hasNext();) {
					CmsProperty element = (CmsProperty) iter.next();
					if(element.getResourceValue()!=null) {
						p.put("resource."+element.getName(),element.getValue());
					}
					if(element.getStructureValue()!=null) {
						p.put("structure."+element.getName(),element.getValue());
					}
				}
				ByteArrayOutputStream baos=new ByteArrayOutputStream();
				p.store(baos,"");
				return baos.toByteArray();
			} catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException("could not read from file ("+e.getMessage()+")");
			}
		} else {
			if(!isFile()) {
				throw new RuntimeException("not a file");
			} else {
				try {
					CmsFile cf;
					co.getRequestContext().setCurrentProject(cp);
					cf = co.readFile(cr.getRootPath());
					return cf.getContents();
				} catch (CmsException e) {
					e.printStackTrace();
					throw new RuntimeException("could not read from file ("+e.getMessage()+")");
				}
			}
		}
	}

	public long lastModified() {
		if(realFile!=null) {
			return realFile.lastModified();
		} else {
			return cr==null?0:cr.getDateLastModified();
		}
	}

	public String getParentPath() {
		if(realFile!=null) {
			return realFile.getParent();
		} else {
			String path=this.path+"/";
			path=ServerPI.normalizePath(path);
			path=path.substring(0,path.lastIndexOf("/"));
			path=ServerPI.normalizePath(path);
			return path;
		}
	}
	
	public void createChild(String name, int type) throws CommandException {
		if(isRoot) throw new CommandException(500,"can't create files or folders here, please go into a project first");
		if(isFile()) throw new CommandException(500,"can't create child of a file '"+name+"'");
		String fullPath=ServerPI.normalizePath(getCmsPath()+"/"+name);
		if(cp.isOnlineProject()) throw new RuntimeException("can't modify an online project");
		try {
			co.getRequestContext().setCurrentProject(cp);
			co.createResource(fullPath,type);
		} catch (Exception e) {
			throw new CommandException(500,"could not create file ("+e.getMessage()+")");
		}
	}
	
	public boolean delete() {
		if(realFile!=null) return true;
		try {
			co.getRequestContext().setCurrentProject(cp);
			
			co.lockResource(cr.getRootPath());
			co.getLock(cr);

			co.deleteResource(getCmsPath(),CmsResource.DELETE_PRESERVE_SIBLINGS);

			return true;
		} catch (Exception ce) {
			ce.printStackTrace();
			return false;

		} finally {
			try {
				co.unlockResource(cr.getRootPath());
			} catch (Exception e) {
			}
		}
	}
	
	public int compareTo(File f) {
		return super.compareTo(f);
	}
	
	public String toString () {
		return "FtpFile at "+this.path+" cp is "+cp+", cr is "+cr;
	}

	public boolean createNewFile(CmsObject co) throws CommandException {
		if (realFile!=null) return realFile.createNewFile(co);
		int type=-1;
		if(path.endsWith(".jsp")) {
			type=CmsResourceTypeJsp.getStaticTypeId();
		} else if(
				path.toLowerCase().endsWith(".gif") ||
				path.toLowerCase().endsWith(".jpeg") ||
				path.toLowerCase().endsWith(".jpg") ||
				path.toLowerCase().endsWith(".png")
				
		) {
			type=CmsResourceTypeImage.getStaticTypeId();
		} else if(
				path.toLowerCase().endsWith(".jar") ||
				path.toLowerCase().endsWith(".class")
		) {
			type=CmsResourceTypeBinary.getStaticTypeId();
		} else if(
				path.toLowerCase().endsWith(".txt") ||
				path.toLowerCase().endsWith(".html") ||
				path.toLowerCase().endsWith(".htm")
		) {
			type=CmsResourceTypePlain.getStaticTypeId();
		} else {
			type=CmsResourceTypeBinary.getStaticTypeId();
		}
		
		
		String p="/"+cp.getName()+"/"+path+"/";

		
		p=ServerPI.normalizePath(p);
		String dir=p.substring(0,p.lastIndexOf("/"));
		String name=p.substring(p.lastIndexOf("/")+1);
		
//		System.err.println("FtpFile.createNewFile() : cp   = "+cp);
//		System.err.println("FtpFile.createNewFile() : path = "+p);
//		System.err.println("FtpFile.createNewFile() : dir  = "+dir);
//		System.err.println("FtpFile.createNewFile() : name = "+name);

		// getting the parent dir;
		FtpFile file=new FtpFile(co,dir);

		try {
			file.createChild(name,type);
			ensureRessource();
			co.getRequestContext().setCurrentProject(cp);
			this.path=file.getCmsPath()+"/"+name;
			this.path=ServerPI.normalizePath(this.path);
			this.cr = co.readResource(this.path);
			return true;
		} catch (CommandException ce) {
			throw ce;
		} catch (Exception e) {
			e.printStackTrace();
			throw new CommandException(500,"could not create new file ("+e.getMessage()+")");
		}
	}
	
}
