package de.ruben.opencms.modules.ftpd;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import de.ruben.opencms.modules.ftpd.Representation;
import de.ruben.opencms.modules.ftpd.TransmissionMode;

/**
 * This class handles transmissions using STREAM mode. The data is sent as a
 * continuous stream of data.
 * 
 * taken from ryan heise's jftpd package: 
 * 	
 * 	http://www.progsoc.uts.edu.au/~rheise/projects/jftpd/
 * 
 * modified by ruben malchow :
 * 
 * 	http://disco.kalkatraz.de/opencmsftpd
 * 
 */

public class StreamTransmissionMode extends TransmissionMode {
	/* size of data chunks sent at a time. */
	private static final int BUFSIZ = 1024;

	StreamTransmissionMode() {
		super('S');
	}

	public void sendFile(InputStream in, Socket s, Representation representation) throws IOException {
		OutputStream out = representation.getOutputStream(s);
		byte buf[] = new byte[BUFSIZ];
		int nread;
		while ((nread = in.read(buf)) > 0) {
			out.write(buf, 0, nread);
		}
		out.close();
	}

	public void receiveFile(Socket s, OutputStream out,Representation representation) throws IOException {
		InputStream in = representation.getInputStream(s);
		byte buf[] = new byte[BUFSIZ];
		int nread;
		while ((nread = in.read(buf, 0, BUFSIZ)) > 0) {
			out.write(buf, 0, nread);
		}
		in.close();
	}
}
