package de.ruben.opencms.modules.ftpd;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Hashtable;

import de.ruben.opencms.modules.ftpd.Representation;
import de.ruben.opencms.modules.ftpd.StreamTransmissionMode;

/**
 * Subclasses of TransmissionMode handle the sending or receiving of files in
 * different transmission modes.
 * 
 * taken from ryan heise's jftpd package: 
 * 	
 * 	http://www.progsoc.uts.edu.au/~rheise/projects/jftpd/
 * 
 * modified by ruben malchow :
 * 
 * 	http://disco.kalkatraz.de/opencmsftpd
 * 
 */

public abstract class TransmissionMode {

	private static Hashtable transmissionModes = new Hashtable();

	public static final TransmissionMode STREAM = new StreamTransmissionMode();

	public static TransmissionMode get(char code) {
		return (TransmissionMode) transmissionModes.get(new Character(code));
	}

	private char code;

	protected TransmissionMode(char code) {
		this.code = code;
		transmissionModes.put(new Character(code), this);
	}

	public final char getCode() {
		return code;
	}

	public abstract void sendFile(InputStream in, Socket s, Representation representation) throws IOException;

	public abstract void receiveFile(Socket s, OutputStream out, Representation representation) throws IOException;
	
}
